using System;
using System.Text;
//using org.bouncycastle.math;
//using org.bouncycastle.math.ec;
//using org.bouncycastle.security;
using org.bouncycastle.crypto;
//using org.bouncycastle.crypto.signers;
using org.bouncycastle.crypto.digests;
//using org.bouncycastle.crypto.engines;
//using org.bouncycastle.crypto.agreement;
//using org.bouncycastle.crypto.generators;
//using org.bouncycastle.crypto.parameters;
//using org.bouncycastle.crypto.macs;
//using org.bouncycastle.crypto.modes;
//using org.bouncycastle.crypto.paddings;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;


namespace test.crypto
{
	/**
	 * RIPEMD160 Digest Test
	 */
	public class RIPEMD160DigestTest: Test
	{
		readonly static String[] messages = 
		{
			"",
			"a",
			"abc",
			"message digest",
			"abcdefghijklmnopqrstuvwxyz",
			"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
			"12345678901234567890123456789012345678901234567890123456789012345678901234567890"
		};

		readonly static String[] digests = {
			"9c1185a5c5e9fc54612808977ee8f548b2258d31",
			"0bdc9d2d256b3ee9daae347be6f4dc835a467ffe",
			"8eb208f7e05d987a9b044a8e98c6b087f15a0bfc",
			"5d0689ef49d2fae572b881b123a85ffa21595f36",
			"f71c27109c692c1b56bbdceb5b9d2865b3708dbc",
			"12a053384a9c0c88e405a06c27dcf49ada62eb2b",
			"b0e20b6e3116640286ed3a87a5713079b21f5189",
			"9b752e45573d4b39f4dbd3323cab82bf63326bfb"
		};

		readonly static String million_a_digest = "52783243c1697bdbe16d37f97f68f08325dc1528";

		public static bool arraysEqual(byte[] a, byte[] b)
		{
			if (a == b) return true;
			if (a.Length != b.Length) return false;

			for (int i = 0; i < a.Length; i++)
			{
				if (a[i] != b[i]) return false;
			}

			return true;
		}
        
		public String getName()
		{
			return "RIPEMD160";
		}

		public TestResult perform()
		{
			Digest digest = new RIPEMD160Digest();
			byte[] resBuf = new byte[digest.getDigestSize()];

			for (int i = 0; i < messages.Length; i++)
			{
				byte[] m = getBytes(messages[i]);
				digest.update(m, 0, m.Length);
				digest.doFinal(resBuf, 0);

				if (!arraysEqual(resBuf, Hex.decode(digests[i])))
				{
					return new SimpleTestResult(false, getName() + ": Vector " + i + " failed");
				}
			}

			//
			// test 2
			//
			byte[] mm = getBytes(messages[messages.Length-1]);

			digest.update(mm, 0, mm.Length/2);

			// clone the Digest
			Digest d = new RIPEMD160Digest((RIPEMD160Digest)digest);

			digest.update(mm, mm.Length/2, mm.Length - mm.Length/2);
			digest.doFinal(resBuf, 0);

			if (!arraysEqual(resBuf, Hex.decode(digests[digests.Length-1])))
			{
				return new SimpleTestResult(false,
					"RIPEMD160 failing clone test"
					+ System.Environment.NewLine
					+ "    expected: " + digests[digests.Length-1]
					+ System.Environment.NewLine
					+ "    got     : " + ByteArray2String(Hex.encode(resBuf)));
			}

			d.update(mm, mm.Length/2, mm.Length - mm.Length/2);
			d.doFinal(resBuf, 0);

			if (!arraysEqual(resBuf, Hex.decode(digests[digests.Length-1])))
			{
				return new SimpleTestResult(false,
					"RIPEMD160 failing clone test - part 2"
					+ System.Environment.NewLine
					+ "    expected: " +  digests[digests.Length-1]
					+ System.Environment.NewLine
					+ "    got     : " + ByteArray2String(Hex.encode(resBuf)));
			}

			for (int i = 0; i < 1000000; i++)
			{
				digest.update((byte)'a');
			}
			digest.doFinal(resBuf, 0);

			if (!arraysEqual(resBuf, Hex.decode(million_a_digest)))
			{
				return new SimpleTestResult(false, getName() + ": Million a's failed");
			}

			return new SimpleTestResult(true, getName() + ": Okay");
		}
		private byte [] getBytes(String str)
		{
			char[]  cs = str.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = Convert.ToByte(cs[i]);
			}
			return bs;
		}
		private String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}


		[STAThread]
		static void Main(string[] args)
		{
			RIPEMD160DigestTest test = new RIPEMD160DigestTest();
			TestResult          result = test.perform();

			Console.WriteLine(result.ToString());
		}
	}
}
