using System;
using System.IO;
using org.bouncycastle.math;

namespace org.bouncycastle.asn1
{
	public class DEREnumerated: DERObject
	{
		internal byte[]      bytes;

		/**
		* return an integer from the passed in object
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DEREnumerated getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DEREnumerated).IsInstanceOfType(obj)))
			{
				return (DEREnumerated)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DEREnumerated(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return an Enumerated from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param explicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DEREnumerated getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}

		public DEREnumerated(
			int         value)
		{
			bytes = BigInteger.valueOf(value).toByteArray();
		}

		public DEREnumerated(
			BigInteger   value)
		{
			bytes = value.toByteArray();
		}

		public DEREnumerated(
			byte[]   bytes)
		{
			this.bytes = bytes;
		}

		public BigInteger getValue()
		{
			return new BigInteger(bytes);
		}

		public override void encode(
			DEROutputStream outStream)
			//throws IOException
		{
			outStream.writeEncoded(DERTags.ENUMERATED, bytes);
		}
	    
		public override bool Equals(Object  o)
		{
			if ((o == null) || !(typeof(DEREnumerated).IsInstanceOfType(o)))
			{
				return false;
			}

			DEREnumerated other = (DEREnumerated)o;

			if (bytes.Length != other.bytes.Length)
			{
				return false;
			}

			for (int i = 0; i != bytes.Length; i++)
			{
				if (bytes[i] != other.bytes[i])
				{
					return false;
				}
			}

			return true;
		}

		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

	}

}