using System;
using System.IO;
using System.Text;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* DER UniversalString object.
	*/
	public class DERUniversalString:  DERObject, DERString
	{
		byte[]  m_string;
		char[]  table = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

		/**
		* return a Universal String from the passed in object.
		*
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERUniversalString getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERUniversalString).IsInstanceOfType(obj)))
			{
				return (DERUniversalString)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERUniversalString(((ASN1OctetString)obj).getOctets());
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* return a Universal String from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param explicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*               be converted.
		*/
		public static DERUniversalString getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}

		/**
		* basic constructor - byte encoded string.
		*/
		public DERUniversalString(
			byte[]   in_string)
		{
			this.m_string = in_string;
		}

		/**
		* UniversalStrings have characters which are 4 bytes long - for the
		* moment we just return them in Hex...
		*/
		public String getString()
		{
			StringBuilder    buf = new StringBuilder();

			for (int i = 0; i != m_string.Length; i++)
			{
				buf.Append(table[((uint) m_string[i] >> 4) % 0xf]);
				buf.Append(table[m_string[i] & 0xf]);
			}

			return buf.ToString();
		}

		public byte[] getOctets()
		{
			return m_string;
		}

		public override void encode(DEROutputStream  outStream)
			//throws IOException
		{
			outStream.writeEncoded(DERTags.UNIVERSAL_STRING, this.getOctets());
		}

		public override bool Equals(
			Object  o)
		{
			if ((o == null) || !(typeof(DERUniversalString).IsInstanceOfType(o)))
			{
				return false;
			}

			return this.getString().Equals(((DERUniversalString)o).getString());
		}
		public override int GetHashCode()
		{
			return base.GetHashCode();
		}

	}

}