using System;
using org.bouncycastle.math;
using org.bouncycastle.asn1;
using org.bouncycastle.asn1.x509;

namespace org.bouncycastle.asn1.pkcs
{
	public class IssuerAndSerialNumber: DEREncodable
	{
		X509Name    name;
		DERInteger  certSerialNumber;

		public static IssuerAndSerialNumber getInstance(
			Object  obj)
		{
			
			if (typeof(IssuerAndSerialNumber).IsInstanceOfType(obj))
			{
				return (IssuerAndSerialNumber)obj;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new IssuerAndSerialNumber((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}

		public IssuerAndSerialNumber(
			ASN1Sequence    seq)
		{
			this.name = X509Name.getInstance(seq.getObjectAt(0));
			this.certSerialNumber = (DERInteger)seq.getObjectAt(1);
		}

		public IssuerAndSerialNumber(
			X509Name    name,
			BigInteger  certSerialNumber)
		{
			this.name = name;
			this.certSerialNumber = new DERInteger(certSerialNumber);
		}

		public IssuerAndSerialNumber(
			X509Name    name,
			DERInteger  certSerialNumber)
		{
			this.name = name;
			this.certSerialNumber = certSerialNumber;
		}

		public X509Name getName()
		{
			return name;
		}

		public DERInteger getCertificateSerialNumber()
		{
			return certSerialNumber;
		}

		public DERObject getDERObject()
		{
			ASN1EncodableVector    v = new ASN1EncodableVector();

			v.add(name);
			v.add(certSerialNumber);

			return new DERSequence(v);
		}
	}

}