using System;
using System.Text;
//using org.bouncycastle.math;
//using org.bouncycastle.math.ec;
//using org.bouncycastle.security;
using org.bouncycastle.crypto;
//using org.bouncycastle.crypto.signers;
using org.bouncycastle.crypto.digests;
//using org.bouncycastle.crypto.engines;
//using org.bouncycastle.crypto.agreement;
//using org.bouncycastle.crypto.generators;
using org.bouncycastle.crypto.parameters;
using org.bouncycastle.crypto.macs;
//using org.bouncycastle.crypto.modes;
//using org.bouncycastle.crypto.paddings;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;


namespace test.crypto
{
	/**
	 * RIPEMD128 HMac Test, test vectors from RFC 2286
	 */
	public class RIPEMD128HMacTest: Test
	{
		readonly static String[] keys = 
		{
			"0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b0b",
			"4a656665",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
			"0102030405060708090a0b0c0d0e0f10111213141516171819",
			"0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c0c",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
			"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
		};

		readonly static String[] digests = {
			"fbf61f9492aa4bbf81c172e84e0734db",
			"875f828862b6b334b427c55f9f7ff09b",
			"09f0b2846d2f543da363cbec8d62a38d",
			"bdbbd7cf03e44b5aa60af815be4d2294",
			"e79808f24b25fd031c155f0d551d9a3a",
			"dc732928de98104a1f59d373c150acbb",
			"5c6bec96793e16d40690c237635f30c5"
		};

		readonly static String[] messages = {
			"Hi There",
			"what do ya want for nothing?",
			"0xdddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddddd",
			"0xcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcdcd",
			"Test With Truncation",
			"Test Using Larger Than Block-Size Key - Hash Key First",
			"Test Using Larger Than Block-Size Key and Larger Than One Block-Size Data"
		};

		public static bool arraysEqual(byte[] a, byte[] b)
		{
			if (a == b) return true;
			if (a.Length != b.Length) return false;

			for (int i = 0; i < a.Length; i++)
			{
				if (a[i] != b[i]) return false;
			}

			return true;
		}
        
		public String getName()
		{
			return "RIPEMD128HMac";
		}

		public TestResult perform()
		{
			HMac hmac = new HMac(new RIPEMD128Digest());
			byte[] resBuf = new byte[hmac.getMacSize()];

			for (int i = 0; i < messages.Length; i++)
			{
				byte[] m = getBytes(messages[i]);
				if (messages[i].StartsWith("0x"))
				{
					m = Hex.decode(messages[i].Substring(2));
				}
				hmac.init(new KeyParameter(Hex.decode(keys[i])));
				hmac.update(m, 0, m.Length);
				hmac.doFinal(resBuf, 0);

				if (!arraysEqual(resBuf, Hex.decode(digests[i])))
				{
					return new SimpleTestResult(false, getName() + ": Vector " + i + " failed");
				}
			}

			return new SimpleTestResult(true, getName() + ": Okay");
		}
		private byte [] getBytes(String str)
		{
			char[]  cs = str.ToCharArray();
			byte[]  bs = new byte[cs.Length];

			for (int i = 0; i != cs.Length; i++)
			{
				bs[i] = Convert.ToByte(cs[i]);
			}
			return bs;
		}
		private String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}

		[STAThread]
		static void Main(string[] args)
		{
			RIPEMD128HMacTest   test = new RIPEMD128HMacTest();
			TestResult          result = test.perform();

			Console.WriteLine(result.ToString());
		}
	}

}
